@@ -23,6 +23,7 @@ __all__ = [
     'BlobField',
     'BooleanField',
     'CharField',
+    'Clause',
     'DateField',
     'DateTimeField',
     'DecimalField',
@@ -102,6 +103,20 @@ if sqlite3:
     sqlite3.register_adapter(datetime.time, str)
     sqlite3.register_converter('decimal', lambda v: decimal.Decimal(v))
 
+SQLITE_DT_FORMATS = (
+    '%Y-%m-%d %H:%M:%S',
+    '%Y-%m-%d %H:%M:%S.%f',
+    '%Y-%m-%d',
+    '%H:%M:%S',
+    '%H:%M:%S.%f',
+    '%H:%M')
+DT_LOOKUPS = set(['year', 'month', 'day', 'hour', 'minute', 'second'])
+
+def _sqlite_date_part(lookup_type, datetime_string):
+    assert lookup_type in DT_LOOKUPS
+    dt = format_date_time(datetime_string, SQLITE_DT_FORMATS)
+    return getattr(dt, lookup_type)
+
 if psycopg2:
     import psycopg2.extensions
     psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
@@ -132,6 +147,7 @@ OP_IN = 26
 OP_IS = 27
 OP_LIKE = 28
 OP_ILIKE = 29
+OP_BETWEEN = 30
 
 DJANGO_MAP = {
     'eq': OP_EQ,
@@ -293,6 +309,14 @@ class Func(Leaf):
 
 fn = Func(None)
 
+class Clause(Leaf):
+    def __init__(self, *pieces):
+        super(Clause, self).__init__()
+        self.pieces = pieces
+
+    def clone_base(self):
+        return Clause(*self.pieces)
+
 
 class FieldDescriptor(object):
     def __init__(self, field):
@@ -334,6 +358,7 @@ class Field(Leaf):
         Field._field_counter += 1
         self._order = Field._field_counter
 
+        self._is_bound = False
         super(Field, self).__init__()
 
     def clone_base(self, **kwargs):
@@ -341,7 +366,9 @@ class Field(Leaf):
            null=self.null,
            index=self.index,
            unique=self.unique,
+           verbose_name=self.verbose_name,
            help_text=self.help_text,
+           db_column=self.db_column,
            default=self.default,
            choices=self.choices,
            primary_key=self.primary_key,
@@ -349,10 +376,9 @@ class Field(Leaf):
            **kwargs
        )
        inst.attributes = dict(self.attributes)
-       inst.name = self.name
-       inst.model_class = self.model_class
-       inst.db_column = self.db_column
-       inst.verbose_name = self.verbose_name
+       if self._is_bound:
+           inst.name = self.name
+           inst.model_class = self.model_class
        return inst
 
     def add_to_class(self, model_class, name):
@@ -363,7 +389,12 @@ class Field(Leaf):
 
         model_class._meta.fields[self.name] = self
         model_class._meta.columns[self.db_column] = self
+
         setattr(model_class, name, FieldDescriptor(self))
+        self._is_bound = True
+
+    def get_database(self):
+        return self.model_class._meta.database
 
     def field_attributes(self):
         return {}
@@ -383,6 +414,9 @@ class Field(Leaf):
     def __hash__(self):
         return hash(self.name + '.' + self.model_class.__name__)
 
+    def between(self, low, high):
+        return Expr(self, OP_BETWEEN, Clause(low, R('AND'), high))
+
 
 class IntegerField(Field):
     db_field = 'int'
@@ -477,6 +511,11 @@ def format_date_time(value, formats, post_process=None):
             pass
     return value
 
+def _date_part(date_part):
+    def dec(self):
+        return self.model_class._meta.database.extract_date(date_part, self)
+    return dec
+
 class DateTimeField(Field):
     db_field = 'datetime'
 
@@ -494,6 +533,13 @@ class DateTimeField(Field):
             return format_date_time(value, self.attributes['formats'])
         return value
 
+    year = property(_date_part('year'))
+    month = property(_date_part('month'))
+    day = property(_date_part('day'))
+    hour = property(_date_part('hour'))
+    minute = property(_date_part('minute'))
+    second = property(_date_part('second'))
+
 class DateField(Field):
     db_field = 'date'
 
@@ -514,6 +560,10 @@ class DateField(Field):
             return value.date()
         return value
 
+    year = property(_date_part('year'))
+    month = property(_date_part('month'))
+    day = property(_date_part('day'))
+
 class TimeField(Field):
     db_field = 'time'
 
@@ -536,6 +586,10 @@ class TimeField(Field):
             return value.time()
         return value
 
+    hour = property(_date_part('hour'))
+    minute = property(_date_part('minute'))
+    second = property(_date_part('second'))
+
 class BooleanField(Field):
     db_field = 'bool'
 
@@ -598,14 +652,12 @@ class ForeignKeyField(IntegerField):
         super(ForeignKeyField, self).__init__(null=null, *args, **kwargs)
 
     def clone_base(self):
-        inst = super(ForeignKeyField, self).clone_base(
+         return super(ForeignKeyField, self).clone_base(
             rel_model=self.rel_model,
+            related_name=self.related_name,
             cascade=self.cascade,
             extra=self.extra,
         )
-        inst.related_name = self.related_name
-        inst.rel_model = self.rel_model
-        return inst
 
     def add_to_class(self, model_class, name):
         self.name = name
@@ -626,6 +678,7 @@ class ForeignKeyField(IntegerField):
 
         setattr(model_class, name, RelationDescriptor(self, self.rel_model))
         setattr(self.rel_model, self.related_name, ReverseRelationDescriptor(self))
+        self._is_bound = True
 
         model_class._meta.rel[self.name] = self
         self.rel_model._meta.reverse_rel[self.related_name] = self
@@ -673,6 +726,7 @@ class QueryCompiler(object):
         OP_IS: 'IS',
         OP_LIKE: 'LIKE',
         OP_ILIKE: 'ILIKE',
+        OP_BETWEEN: 'BETWEEN',
         OP_ADD: '+',
         OP_SUB: '-',
         OP_MUL: '*',
@@ -737,6 +791,14 @@ class QueryCompiler(object):
                 exprs.append(parsed)
                 p.extend(params)
             s = '%s(%s)' % (expr.name, ', '.join(exprs))
+        elif isinstance(expr, Clause):
+            p = []
+            exprs = []
+            for piece in expr.pieces:
+                parsed, params = self.parse_expr(piece, alias_map, conv)
+                exprs.append(parsed)
+                p.extend(params)
+            s = ' '.join(exprs)
         elif isinstance(expr, Param):
             s = self.interpolation
             p = [expr.data]
@@ -902,8 +964,9 @@ class QueryCompiler(object):
             params.extend(h_params)
 
         if query._order_by:
-            order_by, _ = self.parse_expr_list(query._order_by, alias_map)
+            order_by, o_params = self.parse_expr_list(query._order_by, alias_map)
             parts.append('ORDER BY %s' % order_by)
+            params.extend(o_params)
 
         if query._limit or (query._offset and db.limit_max):
             limit = query._limit or db.limit_max
@@ -935,8 +998,9 @@ class QueryCompiler(object):
         parts = ['INSERT INTO %s' % self.quote(model._meta.db_table)]
         sets, params = self.parse_field_dict(query._insert)
 
-        parts.append('(%s)' % ', '.join(s[0] for s in sets))
-        parts.append('VALUES (%s)' % ', '.join(s[1] for s in sets))
+        if sets:
+            parts.append('(%s)' % ', '.join(s[0] for s in sets))
+            parts.append('VALUES (%s)' % ', '.join(s[1] for s in sets))
 
         return ' '.join(parts), params
 
@@ -1081,6 +1145,23 @@ class QueryResultWrapper(object):
             except StopIteration:
                 break
 
+class TuplesQueryResultWrapper(QueryResultWrapper):
+    def __init__(self, model, cursor, meta=None):
+        super(TuplesQueryResultWrapper, self).__init__(model, cursor, meta)
+        conv = []
+        identity = lambda x: x
+        for i in range(len(self.cursor.description)):
+            col = self.cursor.description[i][0]
+            if col in model._meta.columns:
+                field_obj = model._meta.columns[col]
+                conv.append(field_obj.python_value)
+            else:
+                conv.append(identity)
+        self.conv = conv
+
+    def process_row(self, row):
+        return tuple([self.conv[i](col) for i, col in enumerate(row)])
+
 class NaiveQueryResultWrapper(QueryResultWrapper):
     def __init__(self, model, cursor, meta=None):
         super(NaiveQueryResultWrapper, self).__init__(model, cursor, meta)
@@ -1357,7 +1438,7 @@ class RawQuery(Query):
     def execute(self):
         if self._qr is None:
             if self._tuples:
-                ResultWrapper = QueryResultWrapper
+                ResultWrapper = TuplesQueryResultWrapper
             elif self._dicts:
                 ResultWrapper = DictQueryResultWrapper
             else:
@@ -1545,7 +1626,7 @@ class SelectQuery(Query):
         if self._dirty or not self._qr:
             query_meta = None
             if self._tuples:
-                ResultWrapper = QueryResultWrapper
+                ResultWrapper = TuplesQueryResultWrapper
             elif self._dicts:
                 ResultWrapper = DictQueryResultWrapper
             elif self._naive or not self._joins or self.verify_naive():
@@ -1562,6 +1643,9 @@ class SelectQuery(Query):
     def __iter__(self):
         return iter(self.execute())
 
+    def iterator(self):
+        return iter(self.execute().iterator())
+
     def __getitem__(self, value):
         offset = limit = None
         if isinstance(value, slice):
@@ -1799,6 +1883,9 @@ class Database(object):
             qc = self.compiler()
             return self.execute_sql(qc.drop_sequence(seq))
 
+    def extract_date(self, date_part, date_field):
+        return fn.EXTRACT(Clause(date_part, R('FROM'), date_field))
+
 
 class SqliteDatabase(Database):
     limit_max = -1
@@ -1810,7 +1897,9 @@ class SqliteDatabase(Database):
     def _connect(self, database, **kwargs):
         if not sqlite3:
             raise ImproperlyConfigured('sqlite3 must be installed on the system')
-        return sqlite3.connect(database, **kwargs)
+        conn = sqlite3.connect(database, **kwargs)
+        conn.create_function('date_part', 2, _sqlite_date_part)
+        return conn
 
     def get_indexes_for_table(self, table):
         res = self.execute_sql('PRAGMA index_list(%s);' % self.quote(table))
@@ -1821,6 +1910,9 @@ class SqliteDatabase(Database):
         res = self.execute_sql('select name from sqlite_master where type="table" order by name;')
         return [r[0] for r in res.fetchall()]
 
+    def extract_date(self, date_part, date_field):
+        return fn.date_part(date_part, date_field)
+
 
 class PostgresqlDatabase(Database):
     commit_select = True
@@ -1952,6 +2044,10 @@ class MySQLDatabase(Database):
         res = self.execute_sql('SHOW TABLES;')
         return [r[0] for r in res.fetchall()]
 
+    def extract_date(self, date_part, date_field):
+        assert date_part.lower() in DT_LOOKUPS
+        return fn.EXTRACT(Clause(R(date_part), R('FROM'), date_field))
+
 
 class transaction(object):
     def __init__(self, db):
